@@ -140,13 +140,14 @@ start-up modules.
    use PDL::IO::Misc;
    use PDL::IO::FITS;
    use PDL::IO::Pic;
+   use PDL::IO::Storable;
    use PDL::Lvalue;
 
 =cut
 
 
 # set the version:
-$PDL::VERSION = '2.013';
+$PDL::VERSION = '2.014';
 
 # Main loader of standard PDL package
 
@@ -180,6 +181,10 @@ use PDL::IO::Pic;           # rpic/wpic
 
 use PDL::Config;
 
+# Load this to avoid mysterious Storable segfaults
+
+use PDL::IO::Storable;
+
 EOD
 
 die $@ if $@;
